/**
 * @license
 * Copyright 2023 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
export declare const debug: (prefix: string) => ((msg: string, ...param: unknown[]) => void) | undefined;
//# sourceMappingURL=debug.d.ts.map