/**
 * @license
 * Copyright 2023 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
import type { Configuration } from 'puppeteer-core';
/**
 * @internal
 */
export declare const getConfiguration: () => Promise<Configuration>;
//# sourceMappingURL=getConfiguration.d.ts.map