/**
 * Check if the given string is a single complex grapheme. A complex grapheme is one that
 * is made up of multiple characters.
 */
export declare function isSingleComplexGrapheme(value: string): boolean;
/**
 * Check if the given string is a single grapheme.
 */
export declare function isSingleGrapheme(value: string): boolean;
